/*
 * CanData.java
 *
 * Created on 17 August 2007, 13:49
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package ticomViewer;

/**
 *
 * @author karol
 */
public class CanData
{

    public static final int CO_DATA_TYPE_UNKNOWN = -1;
    public static final int CO_NO_DATA = 0x00;  // used, when an entry is a header of the array
    public static final int CO_BOOL1 = 0x01;
    public static final int CO_INT8  = 0x02;
    public static final int CO_INT16 = 0x03;
    public static final int CO_INT32 = 0x04;
    public static final int CO_UINT8 = 0x05;
    public static final int CO_UINT16 = 0x06;
    public static final int CO_UINT32 = 0x07;
    public static final int CO_REAL32 = 0x08;
    public static final int CO_VIS_STR = 0x09;
    public static final int CO_OCTET_STR = 0x0A;
    public static final int CO_UNICODE_STR = 0x0B;
    public static final int CO_TIME_OF_DAY =  0x0C;
    public static final int CO_TIME_DIFF = 0x0D;
    public static final int CO_BIT_STR = 0x0E;
    public static final int CO_DOMAIN =  0x0F;
    public static final int CO_INT24 = 0x10;
    public static final int CO_REAL64 = 0x11;
    public static final int CO_INT40 = 0x12;
    public static final int CO_INT48 = 0x13;
    public static final int CO_INT56 = 0x14;
    public static final int CO_INT64 = 0x15;
    public static final int CO_UINT24 = 0x16;
    public static final int CO_UINT40 = 0x18;
    public static final int CO_UINT48 = 0x19;
    public static final int CO_UINT56 = 0x1A;
    public static final int CO_UINT64 = 0x1B;
    
    public static long bytesArrayToLong(byte[] a)
    {
        long result = 0;
        for( int i = a.length-1; i >= 0; i-- ) result = result * 256 + (a[i] < 0 ? (short)a[i] + 256 : (short)a[i]) ;
        return result;
    }
    
    public static byte[] longToByteArray(long a)
    {
        byte[] b = new byte[8];
        for( int i = 0; i < 8; i++ )
        {
            b[i] = (byte)(a & 0xFF);
            a = a >> 8;
        }
        return b;
    }
    
    public static String convertCanDataToString(byte[] data, int canDataType)
    {
         if( data.length < 1 ) return "no data";
        switch(canDataType)
        {
            case CO_DATA_TYPE_UNKNOWN: return "data type unknown";                
            case CO_NO_DATA: return "no data";
            case CO_BOOL1: return data[0] == 0 ? "false" : "true";
            case CO_INT8:  return Integer.toString(data[0]);
            case CO_INT16: 
            case CO_INT32:
            case CO_UINT8:
            case CO_UINT16:
            case CO_UINT32:
            case CO_INT24:
            case CO_INT40:
            case CO_INT48:
            case CO_INT56:
            case CO_INT64:                
            case CO_UINT24:
            case CO_UINT40:
            case CO_UINT48:
            case CO_UINT56:
            case CO_UINT64:
                    return Long.toString(bytesArrayToLong(data)) + " ( 0x" + Long.toHexString(bytesArrayToLong(data)) + " )";
            
            case CO_REAL32:
            case CO_REAL64:                
                    return "double not supported yet.";
            case CO_VIS_STR:
                    for( int i = 0; i < data.length; i++ ) if( data[i] == 0 ) return new String( data, 0, i);
                    return new String( data );
            case CO_OCTET_STR:
            case CO_UNICODE_STR:
            case CO_TIME_OF_DAY:
            case CO_TIME_DIFF:
            case CO_BIT_STR:
            case CO_DOMAIN:
                return "data type not supported yet" ;                   
        }
        return "unknown CAN data type";
    }
}