/*
 * CanNodeDevice.java
 *
 * Created on 17 August 2007, 13:45
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package ticomViewer;

import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TMode;
import de.desy.tine.structUtils.TTaggedStructure;
import java.awt.Component;
import java.util.Calendar;
import java.util.Date;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;



/**
 *
 * @author karol
 */
public class CanNodeDevice extends Node
{
    private TLink HeartBeatLink;
    private TLink HeartBeatControlLink;  
    private NodeState state;
    private CanNodeIdInfo canNodeIdInfo;
    private PdoMappedVariable[] TpdoVariables = new PdoMappedVariable[128];
    private PdoMappedVariable[] RpdoVariables = new PdoMappedVariable[128];
    private int[] NMTcountersAndTime = new int[10];
    private TLinkCallback hbeatControlCallback = new HbeatControlCallback();
    public static final int NMTREQ_RESET = 129;
    public static final int NMTREQ_PREOPERATIONAL = 128;
    public static final int NMTREQ_OPERATIONAL = 1;
    public static final int NMTREQ_STOP = 2;
    private TLink[] AutomaticVariableLink = new TLink[128];
    private TLinkCallback automaticVariableLinkCallback = new AutomaticVariableLinkCallback();
    private TPDOvariablesTableModel tPdoVariablesTableModel;
    private RPDOvariablesTableModel rPdoVariablesTableModel;
    private FlashProgrammer flashProgrammer;

    void flashProgrammerAssignDevice()
    {
        flashProgrammer.attachLinks(TicomView.TicomViewer.getServerPath(), getName());
    }

    void setProgrammerDisplay( FlashProgrammerDisplayer fpd )
    {
        flashProgrammer.setFlashProgrammerDisplayer(fpd);
        if( fpd != null ) fpd.setDataSource(flashProgrammer);
    }

    private void closeFlashProgrammerLinks()
    {
        flashProgrammer.closeLinks();
    }

    class AutomaticVariableLinkCallback implements TLinkCallback
    {
        @Override
        public void callback(TLink link)
        {
            for( int i = 0; i < TpdoVariables.length; i++ )
            {
                if( link.getProperty().equals(TpdoVariables[i].variableName.toString()))
                {
                    for( int j = 0; j < 8; j++ ) TpdoVariables[i].dataValue[j] = link.getOutputDataObject().getDataBuffer(0)[j];
                    for( int j = 0; j < 8; j++ ) TpdoVariables[i].updateTime[j] = link.getOutputDataObject().getDataBuffer(0)[j+8];
                    tPdoVariablesTableModel.rowChanged(i);
                    return;
                }
            }
        }        
    };
    
    
    class HbeatControlCallback implements TLinkCallback
    {
        @Override
        public void callback(TLink link)
        {
            if( link.getLinkStatus() != 0)
            {
                System.out.println("HBeatControlCallback error: " + link.getLinkStatus() + ", " + link.getError(link.getLinkStatus()) );
                return;
            }
            int h[] = new int[2];
            link.getOutputDataObject().getData(h);
            TicomView.TicomViewer.JLabel_heartbeatEnabled.setText( h[0] != 0 ? "YES" : "NO");
            TicomView.TicomViewer.JLabel_heartbeatPeriod.setText( Integer.toString(h[1]));
        }
    };
    
    CanNodeDevice()
    {
        setName("unknown");
        state = new NodeState();
        state.set(NodeState.BOOT_UP);
        canNodeIdInfo = new CanNodeIdInfo();
        flashProgrammer = new FlashProgrammer();
        for( int i = 0; i < TpdoVariables.length; i++ ) TpdoVariables[i] = new PdoMappedVariable();
        for( int i = 0; i < RpdoVariables.length; i++ ) RpdoVariables[i] = new PdoMappedVariable();
    }
    
    CanNodeDevice(String name, int state)
    {
        setName(name);
        this.state = new NodeState(state);
        canNodeIdInfo = new CanNodeIdInfo();
        flashProgrammer = new FlashProgrammer();
        for( int i = 0; i < TpdoVariables.length; i++ ) TpdoVariables[i] = new PdoMappedVariable();
        for( int i = 0; i < RpdoVariables.length; i++ ) RpdoVariables[i] = new PdoMappedVariable();
    }
    
    public void closeLinks()
    {
        closeAutomaticVariableLinks();
        closeHeartBeatLink();
        closeFlashProgrammerLinks();
    }
    @Override
    public void showControlPanel()
    {
        TicomView.TicomViewer.showCanNodeControlPanel(this);
    }
    
    public String getNodeStateString()
    { return state.toString(); }
    public int getNodeState()
    { return state.get(); }
    public void setNodeState( int st )
    {
        state.set(st);
    }
    int requestNMT(int requestedNMTstate)
    {
        int error;
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/" + getName());
        int[] req = new int[1];
        req[0] = requestedNMTstate;
        TDataType din = new TDataType(req);
        TLink l = new TLink(devname,"NMState",null,din ,TAccess.CA_WRITE);
        if( (error = l.execute(1000)) != 0 )
        {
            System.out.println("Error execute (requestNMT): " + l.getLinkStatus());
        }
        l.close();
        return error;
    }
    
    void attachHeartBeatLink()
    {
        try
        {
            int error;
            String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ getName());
            TDataType dout = new TDataType(NMTcountersAndTime);
            HeartBeatLink = new TLink(devname,"StateStat", dout,null ,TAccess.CA_READ);
            if( HeartBeatLink.attach(TMode.CM_POLL, this,1000 ) < 0 )
            {
                System.out.println("Error attaching link (HeartBeatLinkHandle): " + HeartBeatLink.getLinkStatus());
            }
            
            int[] hbeatState = new int[2];
            dout = new TDataType(hbeatState);
            HeartBeatControlLink = new TLink(devname,"HbeatControl", dout,null,TAccess.CA_READ);
            if( ( HeartBeatControlLink.attach(TMode.CM_POLL, hbeatControlCallback, 1000 )) < 0 )
            {
                System.out.println("Error attaching link (HeartBeatControlLink) : " + HeartBeatControlLink.getLinkStatus());
            }
            return;
        }
        catch( Exception e)
        {
            System.out.println(e);
        }
    }
    
    private void closeHeartBeatLink()
    {
        if( HeartBeatLink != null ) HeartBeatLink.close();
        if( HeartBeatControlLink != null ) HeartBeatControlLink.close();
    }
    
    @Override
    public void callback(TLink link)
    {
        if( link.getLinkStatus() != 0)
        {
            System.out.println("HeartBeat info link error = " + link.getLinkStatus() );
        }
        
        int newState = NMTcountersAndTime[9];
        state.set(newState);
        fireNodeChanged( new NodeChangedEvent());
        TicomView.TicomViewer.JLabel_noHeartBeatTime.setOpaque((newState & NodeState.TIMEOUT) != 0 );
        TicomView.TicomViewer.JLabel_resetEntryTime.setOpaque(newState==NodeState.BOOT_UP);
        TicomView.TicomViewer.JLabel_preoperationalEntryTime.setOpaque(newState==NodeState.PREOPERATIONAL);
        TicomView.TicomViewer.JLabel_operationalEntryTime.setOpaque(newState==NodeState.OPERATIONAL);
        TicomView.TicomViewer.JLabel_stopEntryTime.setOpaque(newState==NodeState.STOPPED);
        
        TicomView.TicomViewer.JLabel_resetEntryTime.repaint();
        TicomView.TicomViewer.JLabel_preoperationalEntryTime.repaint();
        TicomView.TicomViewer.JLabel_operationalEntryTime.repaint();
        TicomView.TicomViewer.JLabel_stopEntryTime.repaint();
        TicomView.TicomViewer.JLabel_noHeartBeatTime.repaint();
        
        TicomView.TicomViewer.JLabel_resetCounter.setText(Integer.toString(NMTcountersAndTime[0]));
        TicomView.TicomViewer.JLabel_resetEntryTime.setText(getTimeStamp((int)NMTcountersAndTime[4]));
        TicomView.TicomViewer.JLabel_preOperationalCounter.setText(Integer.toString(NMTcountersAndTime[1]));
        TicomView.TicomViewer.JLabel_preoperationalEntryTime.setText(getTimeStamp((int)NMTcountersAndTime[5]));
        TicomView.TicomViewer.JLabel_operationalCounter.setText(Integer.toString(NMTcountersAndTime[2]));
        TicomView.TicomViewer.JLabel_operationalEntryTime.setText(getTimeStamp((int)NMTcountersAndTime[6]));
        TicomView.TicomViewer.JLabel_stoppedCounter.setText(Integer.toString(NMTcountersAndTime[3]));
        TicomView.TicomViewer.JLabel_stopEntryTime.setText(getTimeStamp((int)NMTcountersAndTime[7]));
        TicomView.TicomViewer.JLabel_noHeartBeatTime.setText(getTimeStamp((int)NMTcountersAndTime[8]));       
    }
    
    
    static public String getTimeStamp(int seconds)
    {
        if( seconds == 0 ) return "";
        
        Calendar c = Calendar.getInstance();
        c.setTime(new Date((long)seconds*1000));
        String s = new String();
        s = String.format( "%02d", c.get(Calendar.DAY_OF_MONTH)) + "."
                + String.format( "%02d", c.get(Calendar.MONTH)+1) + "."
                + String.format( "%04d", c.get(Calendar.YEAR) )+ ", "
                + String.format( "%02d", c.get(Calendar.HOUR_OF_DAY)) + ":"
                + String.format( "%02d", c.get(Calendar.MINUTE)) +":"
                + String.format( "%02d", c.get(Calendar.SECOND));                      
        return s;
    } 
    
   static public String getTimeStamp(long miliseconds)
    {
        if( miliseconds == 0 ) return "";
        
        Calendar c = Calendar.getInstance();
        c.setTime(new Date(miliseconds));
        String s = new String();
        s = String.format( "%02d", c.get(Calendar.DAY_OF_MONTH)) + "."
                + String.format( "%02d", c.get(Calendar.MONTH)+1) + "."
                + String.format( "%04d", c.get(Calendar.YEAR) )+ ", "
                + String.format( "%02d", c.get(Calendar.HOUR_OF_DAY)) + ":"
                + String.format( "%02d", c.get(Calendar.MINUTE)) +":"
                + String.format( "%02d", c.get(Calendar.SECOND)) + "." 
                +String.format("%d", c.get(Calendar.MILLISECOND));
        
        return s;
    } 
    int setHeartBeatControl(boolean heartBeatOn, int period)
    {
        int error;
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ getName());
        int[] req = new int[2];
        req[0] = heartBeatOn ? 1 : 0;
        req[1] = period;
        TDataType din = new TDataType(req);
        TLink l = new TLink(devname,"HbeatControl",null,din ,TAccess.CA_WRITE);
        if( (error = l.execute(1000)) != 0 )
        {
            System.out.println("Error execute (setHeartBeatControl): " + l.getLinkStatus());
        }
        l.close();
        return error;
    }
    
    public int readNodeIdInfo()
    {
        //read ID data
        int error;
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ getName());
        TLink link = new TLink(devname,"IdInfo",new TDataType(canNodeIdInfo),null,TAccess.CA_READ);
        if( (error = link.execute(1000)) !=0)
        {
            System.out.println("readNodeIdInfo link.execute error: " + error + ", :" + link.getError(error));
        }
        link.close();
        return error;
    }
    
    public CanNodeIdInfo getCanNodeIdInfo()
    {
        return canNodeIdInfo;
    }
    
    public int readNodeData()
    {
        //read PDO mapped data
        int error;
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ getName());
        TLink link = new TLink(devname,"TPDOvariables",new TDataType(TpdoVariables),null,TAccess.CA_READ);
        if( (error = link.execute(1000)) !=0)
        {
            System.out.println("readNodeData (TPDO) link.execute error: " + error + ", :" + link.getError(error));
        }
        link.close();
        fillTpdoTable();
        attachAutomaticVariableLinks();
        
        link = new TLink(devname,"RPDOvariables",new TDataType(RpdoVariables),null,TAccess.CA_READ);
        if( (error = link.execute(1000)) !=0)
        {
            System.out.println("readNodeData (RPDO) link.execute error: " + error + ", :" + link.getError(error));
        } 
        link.close();
        fillRpdoTable();
        
        return error;
    }
    
    private void attachAutomaticVariableLinks()
    {
        for( int i = 0; i < TpdoVariables.length; i++ )
        {
            if( TpdoVariables[i].index[0] == 0 ) return;
            AutomaticVariableLink[i] = attachAutomaticVariableLink(TpdoVariables[i].variableName.toString(), TpdoVariables[i].dataFormat[0]);
        }
    }
    
    private TLink attachAutomaticVariableLink( String varName, int format )
    {
        int error;
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ getName());
        double[] data = new double[2];  // should be long, but TINE does not support 64 long... talk to P.D.
        TDataType dout = new TDataType(data);
        TLink autoVarLink = new TLink(devname,varName,dout,null ,TAccess.CA_READ);
        
        if( (error = autoVarLink.attach( TMode.CM_POLL, automaticVariableLinkCallback, 500 )) < 0 )
        {
            System.out.println("Error: " + error + " attach link (attachAutomaticVariableLink) : " + autoVarLink.getLinkStatus());
        }
        return autoVarLink;
    }
    
    private void fillTpdoTable()
    {                       
        TicomView.TicomViewer.JTable_tPdoVariables.setModel( tPdoVariablesTableModel = new TPDOvariablesTableModel(this,TpdoVariables));       
        TableColumnModel columnModel =  TicomView.TicomViewer.JTable_tPdoVariables.getColumnModel();
        TableColumn buttonColumn = columnModel.getColumn(3);
        buttonColumn.setCellRenderer(new TableCellButtonRenderer(buttonColumn.getCellRenderer()));  
        buttonColumn.setCellEditor(new TableCellButtonEditor(TicomView.TicomViewer.JTable_tPdoVariables, new JCheckBox()));
        buttonColumn.setMaxWidth(200);        
    }
    
    private void fillRpdoTable()
    {       
        TicomView.TicomViewer.JTable_rPdoVariables.setModel( rPdoVariablesTableModel = new RPDOvariablesTableModel(getName(), RpdoVariables));
        TableColumnModel columnModel =  TicomView.TicomViewer.JTable_rPdoVariables.getColumnModel();
        TableColumn buttonColumn = columnModel.getColumn(3);
        buttonColumn.setCellRenderer(new TableCellButtonRenderer(buttonColumn.getCellRenderer()));  
        buttonColumn.setCellEditor(new TableCellButtonEditor(TicomView.TicomViewer.JTable_rPdoVariables, new JCheckBox()));
        buttonColumn.setMaxWidth(200); 
        TableColumn valueColumn = columnModel.getColumn(1);
        valueColumn.setCellEditor(new DefaultCellEditor(new JTextField()) 
            {
                private int currentRow = -1;
                public  int getClickCountToStrat() { return 1; }
                @Override
                public  Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
                {   
                    currentRow = row;
                    return super.getTableCellEditorComponent(table,value,isSelected,row,column);
                }
                @Override
                public  boolean stopCellEditing()
                {
                    rPdoVariablesTableModel.setValueAt("new value, not sent", currentRow ,2);
                    rPdoVariablesTableModel.fireTableCellUpdated(currentRow,2);
                    return super.stopCellEditing();                    
                }              
            });        
    }
    
    private void closeAutomaticVariableLinks()
    {
        for( int i = 0; i < AutomaticVariableLink.length; i++ )
        {
            if( AutomaticVariableLink[i] != null ) AutomaticVariableLink[i].close();
        }
    }
    
    void sdoRead(int index, int subindex )
    {
        int error;
        ODentryDataInfo output = new ODentryDataInfo();
        int[] input = new int[2];
        
        input[0] = index;
        input[1] = subindex;
        
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ getName());
        TLink link = new TLink(devname,"ODentry",new TDataType(output),new TDataType(input),TAccess.CA_READ);
        if( (error = link.execute(1000)) !=0)
        {
            TicomView.TicomViewer.JLabel_odRead.setText("");
            TicomView.TicomViewer.JLabel_odReadStatus.setText("OD read error: " + error + ", " + link.getError(error) );
        }
        else
        {
            TicomView.TicomViewer.JLabel_odRead.setText(CanData.convertCanDataToString(output.data, output.dataCanType[0]));
            TicomView.TicomViewer.JLabel_odReadStatus.setText(link.getLastError());
        }
        link.close();
        return;
    }

    void sdoWrite(int index, int subindex, String data)
    {
        int error;
        ODentryDataInfo input = new ODentryDataInfo();
        input.index[0] = index;
        input.subindex[0] = subindex;
        //input.data = data.getBytes();
        for( int i = 0; i < data.getBytes().length; i++ ) 
                input.data[i] = data.getBytes()[i];  // how to copy??
        input.dataSize[0]=data.length();
        input.dataCanType[0] = CanData.CO_DATA_TYPE_UNKNOWN; // let server to figure out the data type
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ getName());
        TLink link = new TLink(devname,"ODentry",null,new TDataType(input),TAccess.CA_WRITE);
        if( (error = link.execute(1000)) !=0)
        {
            System.out.println("OD write error: " + error + ", :" + link.getError(error) + ", " + link.getLinkStatus());
        }        
        TicomView.TicomViewer.JLabel_odReadStatus.setText(link.getLastError());
        link.close();
        return;
    }
}
class ODentryDataInfo extends TTaggedStructure
{
    public int[] index = new int[1];
    public int[] subindex = new int[1];
    public int[] dataCanType = new int[1];
    public int[] dataSize = new int[1];
    public byte[] data = new  byte[512];

    ODentryDataInfo()
    {
        super("ODENTRYINFO");
        addField(index,"index");
        addField(subindex,"subindex");
        addField(dataCanType,"dataCanType");
        addField(dataSize,"dataSize");
        addField(data,"data");
        initDone();
    }
}

