/*
 * CanopenBusTreeModel.java
 *
 * Created on 20 June 2007, 12:05
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package ticomViewer;


import java.util.EventListener;
import javax.swing.event.EventListenerList;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;


/**
 *
 * @author karol
 */

public class CanopenBusTreeModel extends Node implements TreeModel
{
    private CanDevices canDevices;
    private SyncProducer syncProducer;
    private TimeProducer timeProducer;
    
    private NodeChangedListener nodeChangedListener = new NodeChangedListener()
    {
      @Override
      void nodeChanged(NodeChangedEvent e)
      {
        notifyNodeChanged();
      }       
    };
    
    private void notifyNodeChanged()
    {
     fireNodesChanged(new TreeModelEvent(this, new TreePath(new Object[]{this, getRoot()})));
    }
    public CanopenBusTreeModel()
    {
        setName("Can Bus");    
    }
    
    public void addDefaultDevices()
    {
        add((syncProducer = new SyncProducer()));
        add((timeProducer = new TimeProducer()));
        add(canDevices = new CanDevices());
        
        syncProducer.addNodeChangedListener(nodeChangedListener);
        timeProducer.addNodeChangedListener(nodeChangedListener);
        canDevices.addNodeChangedListener(nodeChangedListener);
         
        fireStructureChanged(new TreeModelEvent(this, new TreePath(new Object[]{this, getRoot()})));
    }
    
    public void removeDevices()
    {
        remove(canDevices);
        remove(syncProducer);
        remove(timeProducer);         
        fireStructureChanged(new TreeModelEvent(this, new TreePath(new Object[]{this, getRoot()})));        
    }
    
    public SyncProducer getSyncProducer()
    { return syncProducer; }
    public TimeProducer getTimeProducer()
    { return timeProducer; }
    public void addCanNodeDevice(CanNodeDevice nd)
    {
        canDevices.add(nd);
        nd.addNodeChangedListener(nodeChangedListener);
    }
    @Override
    public void showControlPanel()
    {
        TicomView.TicomViewer.showBusControlPanel(this);
    }
    @Override
    public String toString()
    {
        return "CANopen bus";
    }
    
    boolean updateBusInfo(BusInfo bi)
    {        
        int unknownState = 0;
        int preoperationalState = 0;
        int operationalState = 0;
        int stoppedState = 0;

        if( syncProducer == null )
        {
            System.out.println( "syncProducer == null");
            return false;
        }
        syncProducer.setOn(bi.syncState[0] > 0); syncProducer.fireNodeChanged( new NodeChangedEvent());
        timeProducer.setOn(bi.timeProducerState[0] > 0 ); timeProducer.fireNodeChanged(new NodeChangedEvent());
        
        int cnt = bi.nNodes[0] - canDevices.getChildCount();
        for( int i = 0; i < cnt; i++ )
        {            
            addCanNodeDevice( new CanNodeDevice( bi.nodeName[i].toString(), bi.nodeState[i]));
            TreePath tp = new TreePath(new Object[]{this, canDevices});           
            fireStructureChanged( new TreeModelEvent(this, tp) );
        }
        
        //count nodes in pre-, operational, stopped, unknown state
        for( int i = 0; i < bi.nNodes[0]; i++)
        {
            ((CanNodeDevice)(canDevices.getChild(i))).setNodeState(bi.nodeState[i]);
            ((CanNodeDevice)(canDevices.getChild(i))).fireNodeChanged(null);
            if((bi.nodeState[i] & BusInfo.HEARTBEAT_TIMEOUT) != 0) ++unknownState;
            else if(bi.nodeState[i] == BusInfo.OPERATIONAL) ++operationalState;
            else if(bi.nodeState[i] == BusInfo.PREOPERATIONAL) ++preoperationalState;
            else if(bi.nodeState[i] == BusInfo.STOPPED) ++stoppedState;
        }
        canDevices.setEffectiveState( preoperationalState, operationalState, stoppedState, unknownState);
        canDevices.fireNodeChanged(null);
        TicomView.TicomViewer.JLabel_totalNumberOfnodes.setText( Integer.toString(bi.nNodes[0]));
        TicomView.TicomViewer.JLabel_totalInPreoperational.setText( Integer.toString(preoperationalState));
        TicomView.TicomViewer.JLabel_totalInOperational.setText( Integer.toString(operationalState));
        TicomView.TicomViewer.JLabel_totalInStopped.setText( Integer.toString(stoppedState));
        TicomView.TicomViewer.JLabel_totalInUnknown.setText( Integer.toString(unknownState));
                
        TicomView.TicomViewer.JLabel_ticomVersion.setText(bi.ticomVersion[0].toString());
        TicomView.TicomViewer.JLabel_ticomStartTime.setText(bi.ticomStartTime[0].toString());
        TicomView.TicomViewer.JLabel_numberOfNodes.setText(Integer.toString(bi.nNodes[0]));
        TicomView.TicomViewer.JLabel_busSpeed.setText(Integer.toString(bi.busSpeed[0]));
        
        return true;
    }
    
    @Override
    public Object getRoot()
    {
        return this;
    }
    
    @Override
    public Object getChild(Object parent, int index)
    {
        return ((Node)parent).getChild(index);
    }
    
    @Override
    public int getChildCount(Object parent)
    {
        return ((Node)parent).getChildCount();
    }
    
    @Override
    public boolean isLeaf(Object node)
    {
        return ((Node)node).isLeaf();
    }
    
    @Override
    public void valueForPathChanged(TreePath path, Object newValue)
    {
        System.out.println("valueForPathChanged("+path+","+newValue+")");
    }
    
    @Override
    public int getIndexOfChild(Object parent, Object child)
    {
        int n = getChildCount(parent);
        for(int i=0; i < n; i++ ) if( getChild(parent,i).equals(child)) return i;
        return -1;
    }
    
    @Override
    public void addTreeModelListener(TreeModelListener l)
    {
        listenerList.add(TreeModelListener.class, l);
    }
    
    @Override
    public void removeTreeModelListener(TreeModelListener l)
    {
        listenerList.remove(TreeModelListener.class, l);
    }
      
    private void fireStructureChanged(TreeModelEvent e)
    {
        EventListener[] listeners = listenerList.getListeners(TreeModelListener.class);
        for(int i = 0; i < listeners.length; i++ ) ((TreeModelListener)listeners[i]).treeStructureChanged(e);
    }
    
    private void fireNodesChanged(TreeModelEvent e)
    {
        EventListener[] listeners = listenerList.getListeners(TreeModelListener.class);
        for(int i = 0; i < listeners.length; i++ ) ((TreeModelListener)listeners[i]).treeNodesChanged(e);
    }
    
    private EventListenerList listenerList = new EventListenerList();
}






        