/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ticomViewer;

import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.structUtils.TTaggedStructure;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.Timer;

/**
 *
 * @author karol
 */

class EmcyData extends TTaggedStructure
{
    public int[]  timeStamp = new int[2];
    public short[] emcyCode = new short[1];   
    public byte[]  errorRegister = new byte[1];    
    public byte[]  manufactureSpecificCode = new byte[5];       
    public byte[]  isMessage = new byte[1];
    public FixedString errorRegisterText = new FixedString(207);
    public FixedString emcyText = new FixedString(48);      
    EmcyData()
    {
        super("EMCY");
        addField(timeStamp, "timeStamp");
        addField(emcyCode, "emcyCode");
        addField(errorRegister, "errorRegister");
        addField(manufactureSpecificCode, "manufactureSpecificCode");
        addField(isMessage, "isMessage");
        addField(errorRegisterText.getBytes(), "errorregisterText");
        addField(emcyText.getBytes(), "emcyText");
        initDone();
    }
}

public class EmcyDisplay implements ActionListener
{

    private JList list;    
    private DefaultListModel model = new DefaultListModel();
    private final static int maxSize = 10;
    private Timer timer;
    private boolean enabled;
    private CanNodeDevice owner;
    int[] timeStamp = new int[2];
        
    EmcyDisplay( JList l )
    {
        timeStamp[0] = 0;
        timeStamp[1] = 1;  
        enabled = false;
        list = l;
        list.setModel(model);
        timer = new Timer(3000, this);
        reset();
        this.owner = null;
    }
    
    void setOwner( CanNodeDevice owner )
    {
        this.owner = owner;
    }
    
    void put( String text )
    {
        if( model.getSize() >= maxSize ) model.remove(maxSize-1);        
        model.add(0, text);
    }    
     
    void reset()
    {  
        disable();
        timeStamp[0] = 0;
        timeStamp[1] = 1;  
        model.clear();      
    }

    void fill( String[] text )
    {
        model.clear();
        int size = text.length > maxSize ? maxSize : text.length;
        for( int i = 0; i < size; i++ ) model.add(0, text[i]);
    }

   
    public void enable()  { timer.setInitialDelay(0); enabled = true; timer.start();   }
    public void disable() { timer.stop();  enabled = false; }
    
    @Override
    public void actionPerformed(ActionEvent e)
    {
        if( !enabled ) return;  
        
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/"+ owner.getName());
        EmcyData emcy = new EmcyData();        
        int error;        
        do
        {  
            TDataType in  = new TDataType( timeStamp );
            TDataType out  = new TDataType( emcy );     
            TLink l = new TLink(devname,"EMCY",out, in,TAccess.CA_READ);  
            if( (error = l.execute(1000)) !=0)
            {               
                put("reading EMCY link.execute error: " + error + ", :" + l.getError(error));
            }
            else 
            {             
                if( emcy.isMessage[0] > 0) 
                {
                    put( CanNodeDevice.getTimeStamp(emcy.timeStamp[0] ) + "." + Long.toString(emcy.timeStamp[1]) 
                        + " Emcy message: " + emcy.emcyText + ", error register = " + emcy.errorRegisterText + " "
                        + " manufacture specific code: {0x" + Integer.toHexString(emcy.manufactureSpecificCode[0]) + " 0x"
                        + Integer.toHexString(emcy.manufactureSpecificCode[1]) + " 0x" 
                        + Integer.toHexString(emcy.manufactureSpecificCode[2]) + " 0x" 
                        + Integer.toHexString(emcy.manufactureSpecificCode[3]) + " 0x" 
                        + Integer.toHexString(emcy.manufactureSpecificCode[4]) + "}");
                    timeStamp[0] = emcy.timeStamp[0];
                    timeStamp[1] = emcy.timeStamp[1]+1;
                }
            }
            l.close();
        } while( error == 0 && emcy.isMessage[0] > 0 ); 
        
        timer.setDelay(3000);        
    }
   
}
