
package ticomViewer;

import de.desy.tine.client.TCallback;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TMode;

/**
 *
 * @author karol
 */
public class FlashProgrammer
{

    private FlashProgrammerDisplayer flashProgrammerDisplayer;
    private TLink statusLink;
    private TLink progressLink;
    private String serverPath;
    private String deviceName;
    private int[] progress = new int[1];
    private int[] status = new int[4];
    private String fileName;

    FlashProgrammer()
    {
        flashProgrammerDisplayer = null;
        serverPath = new String("");
        fileName = new String("");
    }

    public void setFileName(String fileName)
    {
        this.fileName = fileName;
    }

    public void setServerPath(String serverPath)
    {
        this.serverPath = serverPath;
    }

    public void setDeviceName(String deviceName)
    {
        this.deviceName = deviceName;
    }

    public FlashProgrammerDisplayer getFlashProgrammerDisplayer()
    {
        return flashProgrammerDisplayer;
    }

    public void setFlashProgrammerDisplayer(FlashProgrammerDisplayer flashProgrammerDisplayer)
    {
        this.flashProgrammerDisplayer = flashProgrammerDisplayer;
    }

    void attachLinks( String serverPath, String deviceName )
    {
        setServerPath(serverPath);
        setDeviceName(deviceName);

        if( statusLink != null ) statusLink.close();
        statusLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM.STATUS", new TDataType(status), null, TAccess.CA_READ );
        if( ( statusLink.attach(TMode.CM_DATACHANGE, new StatusCallback(), 100)) < 0)
            System.out.println("attach failed for property REPROGRAM.STATUS: " + statusLink.getLinkStatus() + ", " +  statusLink.getLastError());

        if( progressLink != null ) progressLink.close();
        progressLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM", new TDataType(progress), null, TAccess.CA_READ );
        if( ( progressLink.attach(TMode.CM_DATACHANGE, new ProgressCallback(), 100)) < 0)
            System.out.println("attach failed for property REPROGRAM: " + progressLink.getLinkStatus() + ", " +  progressLink.getLastError());

        return;
    }

    void closeLinks()
    {
        statusLink.close();
        progressLink.close();
    }

    void eraseAndReprogram()
    {
        int error = 0;
        int[] req = new int[1];
        req[0] = 1;
        TLink controlLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM.ERASEANDPROG", null,  new TDataType(req),  TAccess.CA_WRITE );
        if( (error = controlLink.execute()) != 0 )
        {
            System.out.println( "--ERROR: flash programmer, erase and reprogramming for device: " +  serverPath + "/" + deviceName + ", error = " + error );
            System.out.println( TErrorList.getErrorString(error));
        }
        controlLink.close();
    }

    void requestControlSums(int[] sums)
    {
        int error = 0;        
        int[] req = new int[1];
        req[0] = 1;
        TLink controlLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM.SUMS", new TDataType(sums),  new TDataType(req),  TAccess.CA_WRITE );
        if( (error = controlLink.execute()) != 0 )
        {
            System.out.println( "--ERROR: requesting control sums device: " +  serverPath + "/" + deviceName + ", error = " + error );
            System.out.println( TErrorList.getErrorString(error));
        }
        controlLink.close();
    }

    void sendData( String fileName )
    {
        int error = 0;
        TLink controlLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM.SENDDATA", null,  new TDataType(fileName),  TAccess.CA_WRITE );
        if( (error = controlLink.execute()) != 0 )
        {
            System.out.println( "--ERROR: flash programmer, sending data for: " +  serverPath + "/" + deviceName + ", error = " + error );
            System.out.println( TErrorList.getErrorString(error));
        }
        controlLink.close();
    }


    void start()
    {
        int error;

        TLink controlLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM", null,  new TDataType(fileName),  TAccess.CA_WRITE );
        if( (error = controlLink.execute(300)) != 0 )
        {
            System.out.println( "--ERROR: starting the flash programmer for: " +  serverPath + "/" + deviceName + ", error = " + error );
            System.out.println( TErrorList.getErrorString(error));
        }
        controlLink.close();
    }

    void switchToMainProgram()
    {
        int error = 0;
        int req[] = new int[1];
        req[0] = 1;
        TLink controlLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM.SWITCHTOMAIN", null,  new TDataType(req),  TAccess.CA_WRITE );
        if( (error = controlLink.execute()) != 0 )
        {
            System.out.println( "--ERROR: flash programmer, switching to main for: " +  serverPath + "/" + deviceName + ", error = " + error );
            System.out.println( TErrorList.getErrorString(error));
        }
        controlLink.close();
    }

    void switchToProgrammingMode()
    {
        int error = 0;
        int req[] = new int[1];

        req[0] = 1;
        TLink controlLink = new TLink(serverPath +"/"+deviceName,"REPROGRAM.SWITCHTOPROGRAM", null,  new TDataType(req),  TAccess.CA_WRITE );
        if( (error = controlLink.execute()) != 0 )
        {
            System.out.println( "--ERROR: flash programmer, switching to main for: " +  serverPath + "/" + deviceName + ", error = " + error );
            System.out.println( TErrorList.getErrorString(error));
        }
        controlLink.close();
    }


    private void updateStatus(int[] status)
    {
       flashProgrammerDisplayer.updateStatus(status);
    }

    private void updateProgress(int progress)
    {
       flashProgrammerDisplayer.updateProgress(progress);
    }

    private class StatusCallback implements TCallback
    {
        public void callback(int LinkIndex, int LinkStatus)
        {
           if( LinkStatus != 0 ) System.out.println("--ERROR: StatusCallback error: " + LinkStatus );
           else updateStatus(status);
        }
    }
    private class ProgressCallback implements TCallback
    {
        public void callback(int LinkIndex, int LinkStatus)
        {
           if( LinkStatus != 0 ) System.out.println("--ERROR: ProgressCallback error: " + LinkStatus );
           else updateProgress(progress[0]);
        }
    }
}
