package ticomViewer;




import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.chart.AcopGraphStyleEnum;
import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.definitions.TMode;
import java.awt.Color;
import java.util.Date;

/*
 * PlotPanel.java
 *
 * Created on 16 August 2007, 13:23
 */

/**
 *
 * @author  karol
 */
public class PlotPanel extends javax.swing.JFrame
{
    int[] value = new int[1];
    
    static class deviceSelection
    {
        private PdoMappedVariable v;
        private String serverPath;
        deviceSelection(String serverPath, PdoMappedVariable v)
        {
            this.serverPath  = serverPath;
            this.v = v;
        }
        public String getServerPath()
        {
            return serverPath;
        }
        public PdoMappedVariable getV()
        {
            return v;
        }
    }
   
   
   class DataHandler implements TLinkCallback
   {

        private int hDisplay;       
        public void callback(TLink link)
        { 
            
            double[] now = new double[1];        
            now[0] = new Date().getTime()/1000.0;            
            switch(ds.getV().dataFormat[0]-512)
            {
                case TFormat.CF_BYTE:                
                case TFormat.CF_SHORT:
                case TFormat.CF_LONG: 
                    int [] v = new int[1];
                    long vl;
                    byte[] dataBuffer = link.getOutputDataObject().getDataBuffer();
                    vl = CanData.bytesArrayToLong(dataBuffer); 
                    v[0] = (int)vl;
                    if( v[0] > Acop_plot.getYMax() ) Acop_plot.setYMax(v[0]);
                    if( v[0] < Acop_plot.getYMin() ) Acop_plot.setYMin(v[0]);
                    Acop_plot.appendScreen(v, hDisplay, now );
                    JLabel_currentValue.setText(Long.toString(vl));
                    break;
                case TFormat.CF_FLOAT:                            
                    float[] fv = new float[1];
                    link.getOutputDataObject().getData(fv);
                    if( fv[0] > Acop_plot.getYMax() ) Acop_plot.setYMax(fv[0]);
                    if( fv[0] < Acop_plot.getYMin() ) Acop_plot.setYMin(fv[0]);
                    Acop_plot.appendScreen(fv[0], hDisplay, now );
                    JLabel_currentValue.setText(Float.toString(fv[0]));
                    break;
                case TFormat.CF_DOUBLE:            
                    double[] dv = new double[1];
                    link.getOutputDataObject().getData(dv);
                    if( dv[0] > Acop_plot.getYMax() ) Acop_plot.setYMax(dv[0]);
                    if( dv[0] < Acop_plot.getYMin() ) Acop_plot.setYMin(dv[0]);
                    Acop_plot.appendScreen(dv[0], hDisplay, now );
                    JLabel_currentValue.setText(Double.toString(dv[0]));
                    break;
            } 
            JLabel_updatetAt.setText(CanNodeDevice.getTimeStamp(link.getLastTimeStamp()));
        }
        
        DataHandler(int hDisplay)
        {
            this.hDisplay = hDisplay;
        }
       
   }
   
    private byte[]  byteRead;
    private short[] shortRead;
    private int[] intRead;
    private float[] floatRead;
    private double[] doubleRead;
    private deviceSelection ds;
    private TPDOvariablesTableModel windowCreator;
    private TLink link;
    
    /** Creates new form PlotPanel */
   
    public PlotPanel(TPDOvariablesTableModel windowCreator, deviceSelection ds)
    {
        this.ds = ds;
        this.windowCreator = windowCreator;
        
        initComponents();        
        displayVariableInfo();
        setTitle("Plot of " + ds.getV().variableName);
        double[] data = new double[1];  // should be long, but TINE does not support 64 long... talk to P.D.
        TDataType dout = new TDataType(data);
        String devname = new String(ds.getServerPath());       
        link = new TLink(devname, ds.getV().variableName.toString(), dout,null ,TAccess.CA_READ);
        int hDisplay = configureAcopPlot(ds.getV().dataFormat[0]);
        if( (link.attach(TMode.CM_POLL, new DataHandler(hDisplay),1000 )) < 0 )
            {
                System.out.println("Error : link.attach" + link.getLinkStatus());
            }                        
        return;                         
    }
        
    void displayVariableInfo()
    {
        JLabel_index.setText("0x"+Integer.toString(ds.getV().index[0],16));
        JLabel_subindex.setText("0x"+Integer.toString(ds.getV().subindex[0],16));
        JLabel_dataFormat.setText(getFormatName(ds.getV().dataFormat[0]));       
        JLabel_plotCaption.setText( ds.getServerPath() + "/" + ds.getV().variableName );
    }
    
    int configureAcopPlot(int dataFormat )
    {
        Acop_plot.setGraphStyle(AcopGraphStyleEnum.TimeLin.ordinal()); 
        Acop_plot.setDrawStyle(AcopDisplayMode.PolyLine.ordinal());
        Acop_plot.setXMin( new Date().getTime()/1000.0 );
        Acop_plot.setXMax( Acop_plot.getXMin() + 30 );
        Acop_plot.setCaption("");
        Acop_plot.setYMin(0);
        Acop_plot.setYMax(10);
        Acop_plot.setYBestScale(true);
        Acop_plot.setYGrid(true);
        Acop_plot.setXGrid(true);
        Acop_plot.setDisplayMode(AcopDisplayMode.PolyLine.ordinal()); // any constant defined in ACOP??
        Acop_plot.setDrawWidth(1);
        Acop_plot.setGridColor( new Color(246,245,240));
        Acop_plot.setBackground(new Color(59,45,124));
        Acop_plot.setFrameForeColor( Acop_plot.getGridColor());
        Acop_plot.setForeground( new Color(153,255,0) );
        switch(dataFormat - 512)
        {
            case TFormat.CF_BYTE:
                return Acop_plot.draw( new byte[1000], new byte[1000], true);
            case TFormat.CF_SHORT:
                return Acop_plot.draw( new short[1000], new short[1000], true);
            case TFormat.CF_LONG:
                return Acop_plot.draw( new int[1000], new int[1000], true);                
            case TFormat.CF_FLOAT:
                return Acop_plot.draw( new float[1000], new float[1000], true);                
            case TFormat.CF_DOUBLE:
                return Acop_plot.draw( new double[1000], new double[1000], true);                
            default:
                System.out.println( "Warning: configureAcopPlot("+dataFormat+"), unsupported format" );
                break;
        }
        return -1;
    }
    
    
    private String getFormatName( int format )
    {
         switch(format-512)
         {
            case TFormat.CF_BYTE:
                return "BYTE";
            case TFormat.CF_SHORT:
                return "SHORT";
            case TFormat.CF_LONG:
                return "INTEGER32";
            case TFormat.CF_FLOAT:
                return "FLOAT";
            case TFormat.CF_DOUBLE:
                return "DOUBLE";            
        }
         return "Unknown";
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        JPanel_top = new javax.swing.JPanel();
        JLabel_plotCaption = new javax.swing.JLabel();
        JPanel_central = new javax.swing.JPanel();
        Acop_plot = new de.desy.acop.displayers.AcopChart();
        JPanel_left = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        JLabel_index = new javax.swing.JLabel();
        JLabel_subindex = new javax.swing.JLabel();
        JLabel_dataFormat = new javax.swing.JLabel();
        JLabel_currentValue = new javax.swing.JLabel();
        JLabel_updatetAt = new javax.swing.JLabel();
        JButton_print = new javax.swing.JButton();
        JPanel_bottom = new javax.swing.JPanel();
        JSlider_timeRange = new javax.swing.JSlider();
        jLabel3 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                formWindowClosed(evt);
            }
        });

        JPanel_top.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));

        JLabel_plotCaption.setBackground(new java.awt.Color(204, 204, 255));
        JLabel_plotCaption.setFont(new java.awt.Font("Arial", 1, 12));
        JLabel_plotCaption.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        JLabel_plotCaption.setText("jLabel1");
        JLabel_plotCaption.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        JLabel_plotCaption.setOpaque(true);

        javax.swing.GroupLayout JPanel_topLayout = new javax.swing.GroupLayout(JPanel_top);
        JPanel_top.setLayout(JPanel_topLayout);
        JPanel_topLayout.setHorizontalGroup(
            JPanel_topLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(JLabel_plotCaption, javax.swing.GroupLayout.DEFAULT_SIZE, 652, Short.MAX_VALUE)
        );
        JPanel_topLayout.setVerticalGroup(
            JPanel_topLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(JLabel_plotCaption, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 26, Short.MAX_VALUE)
        );

        JPanel_central.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));

        Acop_plot.addAcopListener(new de.desy.acop.chart.AcopListener() {
            public void acopMouseClick(de.desy.acop.chart.AcopEvent evt) {
            }
            public void acopMouseDoubleClick(de.desy.acop.chart.AcopEvent evt) {
            }
            public void acopMouseMove(de.desy.acop.chart.AcopEvent evt) {
            }
            public void acopMouseEnter(de.desy.acop.chart.AcopEvent evt) {
            }
            public void acopMouseExit(de.desy.acop.chart.AcopEvent evt) {
            }
            public void acopMouseZoom(de.desy.acop.chart.AcopEvent evt) {
            }
            public void acopMousePressed(de.desy.acop.chart.AcopEvent evt) {
            }
            public void acopReceive(de.desy.acop.chart.AcopEvent evt) {
                Acop_plotAcopReceive(evt);
            }
        });

        javax.swing.GroupLayout JPanel_centralLayout = new javax.swing.GroupLayout(JPanel_central);
        JPanel_central.setLayout(JPanel_centralLayout);
        JPanel_centralLayout.setHorizontalGroup(
            JPanel_centralLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPanel_centralLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(Acop_plot, javax.swing.GroupLayout.DEFAULT_SIZE, 377, Short.MAX_VALUE)
                .addContainerGap())
        );
        JPanel_centralLayout.setVerticalGroup(
            JPanel_centralLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPanel_centralLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(Acop_plot, javax.swing.GroupLayout.DEFAULT_SIZE, 338, Short.MAX_VALUE)
                .addContainerGap())
        );

        JPanel_left.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));

        jLabel1.setText("Index:");

        jLabel2.setText("Subindex:");

        jLabel4.setText("Data format:");

        jLabel5.setText("Current value:");

        jLabel6.setText("Updated at:");

        JLabel_index.setText("0");

        JLabel_subindex.setText("0");

        JLabel_dataFormat.setText("0");

        JLabel_currentValue.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        JLabel_currentValue.setText("0");
        JLabel_currentValue.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));

        JLabel_updatetAt.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        JLabel_updatetAt.setText("0");
        JLabel_updatetAt.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));

        JButton_print.setText("Print");
        JButton_print.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JButton_printActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPanel_leftLayout = new javax.swing.GroupLayout(JPanel_left);
        JPanel_left.setLayout(JPanel_leftLayout);
        JPanel_leftLayout.setHorizontalGroup(
            JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPanel_leftLayout.createSequentialGroup()
                .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPanel_leftLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jLabel1)
                            .addComponent(jLabel5)
                            .addComponent(jLabel4)
                            .addComponent(jLabel6)
                            .addComponent(jLabel2))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLabel_subindex, javax.swing.GroupLayout.PREFERRED_SIZE, 45, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLabel_index, javax.swing.GroupLayout.DEFAULT_SIZE, 153, Short.MAX_VALUE)
                            .addComponent(JLabel_dataFormat, javax.swing.GroupLayout.DEFAULT_SIZE, 153, Short.MAX_VALUE)
                            .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                .addComponent(JLabel_currentValue, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(JLabel_updatetAt, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 153, Short.MAX_VALUE))))
                    .addGroup(JPanel_leftLayout.createSequentialGroup()
                        .addGap(95, 95, 95)
                        .addComponent(JButton_print)))
                .addContainerGap())
        );
        JPanel_leftLayout.setVerticalGroup(
            JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPanel_leftLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(JLabel_index))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(JLabel_subindex))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(JLabel_dataFormat))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(JLabel_currentValue))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPanel_leftLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(JLabel_updatetAt))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 213, Short.MAX_VALUE)
                .addComponent(JButton_print)
                .addContainerGap())
        );

        JPanel_bottom.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));

        JSlider_timeRange.setMajorTickSpacing(600);
        JSlider_timeRange.setMaximum(3600);
        JSlider_timeRange.setMinorTickSpacing(30);
        JSlider_timeRange.setPaintLabels(true);
        JSlider_timeRange.setPaintTicks(true);
        JSlider_timeRange.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                JSlider_timeRangeStateChanged(evt);
            }
        });

        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel3.setText("X-axis time range: ");

        javax.swing.GroupLayout JPanel_bottomLayout = new javax.swing.GroupLayout(JPanel_bottom);
        JPanel_bottom.setLayout(JPanel_bottomLayout);
        JPanel_bottomLayout.setHorizontalGroup(
            JPanel_bottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPanel_bottomLayout.createSequentialGroup()
                .addGap(38, 38, 38)
                .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 143, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(54, 54, 54)
                .addComponent(JSlider_timeRange, javax.swing.GroupLayout.DEFAULT_SIZE, 407, Short.MAX_VALUE)
                .addContainerGap())
        );
        JPanel_bottomLayout.setVerticalGroup(
            JPanel_bottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPanel_bottomLayout.createSequentialGroup()
                .addGroup(JPanel_bottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPanel_bottomLayout.createSequentialGroup()
                        .addGap(29, 29, 29)
                        .addComponent(jLabel3))
                    .addComponent(JSlider_timeRange, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 54, Short.MAX_VALUE))
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(JPanel_top, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addComponent(JPanel_left, javax.swing.GroupLayout.PREFERRED_SIZE, 245, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JPanel_central, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .addComponent(JPanel_bottom, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(JPanel_top, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPanel_central, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JPanel_left, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPanel_bottom, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void JButton_printActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_JButton_printActionPerformed
    {//GEN-HEADEREND:event_JButton_printActionPerformed
        Color grid = Acop_plot.getGridColor();
        Color bckg = Acop_plot.getBackground();
        Color frame = Acop_plot.getFrameForeColor();
        Color fgrd = Acop_plot.getForeground();
        String caption = Acop_plot.getCaption();
        
        Acop_plot.setCaption(ds.getV().variableName.toString());
        Acop_plot.setGridColor(Color.DARK_GRAY);
        Acop_plot.setBackground(Color.WHITE);
        Acop_plot.setFrameForeColor(Color.DARK_GRAY);
        Acop_plot.setForeground(Color.BLACK);
        
        Acop_plot.printScreen();
        
        Acop_plot.setGridColor(grid);
        Acop_plot.setBackground(bckg);
        Acop_plot.setFrameForeColor(frame);
        Acop_plot.setForeground(fgrd);
        Acop_plot.setCaption(caption);   
    }//GEN-LAST:event_JButton_printActionPerformed

    private void formWindowClosed(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowClosed
    {//GEN-HEADEREND:event_formWindowClosed
        if( link!= null ) link.close();
        windowCreator.plotClosed(this);
    }//GEN-LAST:event_formWindowClosed

    private void JSlider_timeRangeStateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_JSlider_timeRangeStateChanged
    {//GEN-HEADEREND:event_JSlider_timeRangeStateChanged
        int t = JSlider_timeRange.getValue();
        if( t < 3 ) t = 3;
        Acop_plot.setXMax( JSlider_timeRange.getValue() + Acop_plot.getXMin());
    }//GEN-LAST:event_JSlider_timeRangeStateChanged

    private void Acop_plotAcopReceive(de.desy.acop.chart.AcopEvent evt)//GEN-FIRST:event_Acop_plotAcopReceive
    {//GEN-HEADEREND:event_Acop_plotAcopReceive
        System.out.println("Acop_plotAcopReceive: " + evt);
    }//GEN-LAST:event_Acop_plotAcopReceive
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    public de.desy.acop.displayers.AcopChart Acop_plot;
    public javax.swing.JButton JButton_print;
    private javax.swing.JLabel JLabel_currentValue;
    private javax.swing.JLabel JLabel_dataFormat;
    private javax.swing.JLabel JLabel_index;
    public javax.swing.JLabel JLabel_plotCaption;
    private javax.swing.JLabel JLabel_subindex;
    private javax.swing.JLabel JLabel_updatetAt;
    public javax.swing.JPanel JPanel_bottom;
    public javax.swing.JPanel JPanel_central;
    public javax.swing.JPanel JPanel_left;
    public javax.swing.JPanel JPanel_top;
    public javax.swing.JSlider JSlider_timeRange;
    public javax.swing.JLabel jLabel1;
    public javax.swing.JLabel jLabel2;
    public javax.swing.JLabel jLabel3;
    public javax.swing.JLabel jLabel4;
    public javax.swing.JLabel jLabel5;
    public javax.swing.JLabel jLabel6;
    // End of variables declaration//GEN-END:variables
    
}
