/*
 * RPDOvariablesTableModel.java
 *
 * Created on 17 August 2007, 13:35
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package ticomViewer;

import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TFormat;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import javax.swing.JButton;
import javax.swing.table.AbstractTableModel;

/**
 *
 * @author karol
 */
public class RPDOvariablesTableModel extends AbstractTableModel implements ActionListener
{    
    private PdoMappedVariable[] pmv;
    private Mbutton[] button;
    private String[] variableStatus;
    private String deviceName;
    
    public boolean isCellEditable(int rowIndex, int colIndex) 
    {
        return colIndex == 1 || colIndex == 3;
    }
    public JButton[] getButtons() 
    {
        return button;
    }
    public RPDOvariablesTableModel(String deviceName, PdoMappedVariable pmv[]) 
    {
        this.pmv = pmv;
        this.deviceName = deviceName;
        
        button = new Mbutton[pmv.length];
        variableStatus = new String[pmv.length];
        
        for(int i = 0; i < button.length; i++ ) 
        {
            String caption = "send";
            button[i] = new Mbutton(caption,i);                   
            button[i].addActionListener(this); 
        }
        for(int i = 0; i < variableStatus.length; i++ ) 
        {
            variableStatus[i] = "";
        }
        
    }
    public void rowChanged( int row )
    { 
        fireTableRowsUpdated(row,row);
    }
    public int getRowCount()
    {
        for( int i = 0; i < pmv.length; i++ ) if( pmv[i].index[0] == 0  ) return i;
        return pmv.length;
    }
    public int getColumnCount()
    {
        return 4;
    }
    public Object getValueAt(int rowIndex, int columnIndex)
    {
        switch( columnIndex )
        {
            case 0: return pmv[rowIndex].variableName.toString();
            case 1: return CanData.bytesArrayToLong( pmv[rowIndex].dataValue );
            case 2: return variableStatus[rowIndex];
            case 3: return button[rowIndex];
        }
        return null;
    }
    
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        if( columnIndex == 1 )
        {
            byte[] b = CanData.longToByteArray(parseLong(value.toString()));
            for( int i = 0; i < b.length; i++ ) pmv[rowIndex].dataValue[i] = b[i];
            return;
        }
        if( columnIndex == 2 )  variableStatus[rowIndex] = value.toString();            
    }
    private String ColumnHeaders[] = {"Variable", "Value", "Variable status", "Action" };

    public String getColumnName(int columnNo)
    {
        return ColumnHeaders[columnNo];
    }      

    public void actionPerformed(ActionEvent e)
    {
      int index = ((Mbutton)(e.getSource())).getIndex();
      if( sendValue(pmv[index]) == 0 ) 
      {
          setValueAt("sent at: " + CanNodeDevice.getTimeStamp(new Date().getTime()), index ,2);
          fireTableCellUpdated(index,2);
      }
    }
    
    public int sendValue( PdoMappedVariable p )
    {
        int error;
        TDataType din;
        String devname = new String(TicomView.TicomViewer.getServerPath() +"/" + deviceName);
        switch( p.dataFormat[0] - 512 )
        {
            case TFormat.CF_BYTE:
                din = new TDataType( new byte[] {p.dataValue[0]} );
                break;
            case TFormat.CF_SHORT:
                din = new TDataType( (short) CanData.bytesArrayToLong(p.dataValue) );
                break;
            case TFormat.CF_LONG:
                int[] ia = new int[1];
                ia[0] = (int)CanData.bytesArrayToLong(p.dataValue);
                din = new TDataType(ia);
                break; 
            default:
                System.out.println("sendValue : unsupported format: " +  p.dataFormat[0]  );
                din = null;
                break;
        }
        TLink link = new TLink(devname,p.variableName.toString(),null,din,TAccess.CA_WRITE);
        if( (error = link.execute(1000)) !=0)
        {
            System.out.println("sendValue " + error + ", :" + link.getError(error) + ", " + link.getLinkStatus());
        } 
        link.close();
        return error;
    }

    private long parseLong(String string)
    {
        if( string.length() < 3 ) return Long.parseLong(string);
        String prefix = string.substring(0,2);
        if( prefix.equalsIgnoreCase("0x")) return Long.parseLong(string.substring(2),16);
        return Long.parseLong(string);
    }
}

